/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.tomcat.util.net.jsse;

import org.apache.tomcat.util.net.AbstractEndpoint;
import org.apache.tomcat.util.net.ServerSocketFactory;

public class TesterBug50640SslImpl extends JSSEImplementation {

    public static final String PROPERTY_NAME = "bug50640";
    public static final String PROPERTY_VALUE = "pass";

    @Override
    public ServerSocketFactory getServerSocketFactory(
            AbstractEndpoint<?> endpoint)  {

        // Check the custom attribute is visible & correcly set
        String flag = endpoint.getProperty(PROPERTY_NAME);
        if (PROPERTY_VALUE.equals(flag)) {
            return super.getServerSocketFactory(endpoint);
        } else {
            return null;
        }
    }

}
